from flask import Flask, request, send_file, Response
import requests
import io
import base64
import logging
import time
from openai import OpenAI

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)

app = Flask(__name__)

# API配置
SILICONFLOW_API_KEY = "sk-kxwsrzianqfxsebnihblrgyyytrrtgvvdjvdiujcuvwymrfp"
TTS_API_URL = "https://api.siliconflow.cn/v1/audio/speech"
xh=0
analysis_len=0
analysis_result=""
data_result=[]
client = OpenAI(
    api_key=SILICONFLOW_API_KEY,
    base_url="https://api.siliconflow.cn/v1",
    timeout=30  # 添加OpenAI客户端超时
)

@app.route('/upload', methods=['POST'])
def handle_image():
    global xh,data_result
    start_time = time.time()
    try:
        # 1. 接收并验证图像数据
        if not request.data:
            logging.warning("收到空请求体")
            return Response("No image data", status=400)
        
        image_data = request.data
        datalen=len(image_data)
        if datalen>20:
            logging.info(f"收到图像数据: {len(image_data)}字节")

            # 2. 快速验证JPEG格式
            if image_data[0:2] != b'\xff\xd8':
                logging.warning("无效的JPEG文件头")
                return Response("Invalid JPEG format", status=400)

            # 3. 图像转base64（限制处理时间）
            encode_start = time.time()
            base64_image = base64.b64encode(image_data).decode('utf-8')
            logging.info(f"Base64编码耗时: {time.time()-encode_start:.2f}s")

            # 4. 调用视觉模型（带超时保护）
            vision_start = time.time()
            response = client.chat.completions.create(
                model="Qwen/Qwen2-VL-72B-Instruct",
                messages=[{
                    "role": "user",
                    "content": [
                        {"type": "image_url", "image_url": {"url": f"data:image/jpeg;base64,{base64_image}"}},
                        {"type": "text", "text": "请详细描述图片中的内容"}
                    ]
                }],
                stream=False,
                timeout=20  # 视觉模型单独超时
            )
            vision_time = time.time() - vision_start
            logging.info(f"视觉分析耗时: {vision_time:.2f}s")
      
            # 5. 提取描述文本
            analysis_result = response.choices[0].message.content
            
            data_result=analysis_result.split("。")
            logging.info(f"分析结果: {data_result[0]},0")
            xh=0
        else:
            print(image_data)
            if(image_data==b'photo'):
              data_result=["正在拍照",0]
              xh=0
            else:
             xh=xh+1
             if len(data_result)>xh+1:
              logging.info(f"分析结果: {data_result[xh]},{xh}") 
            
            
        
        if len(data_result)==xh+1:
           return Response("end", status=400)
        # 6. 生成语音（带重试机制）
        tts_start = time.time()
        audio_data = None
        for attempt in range(3):
                try:
                    audio_data = siliconflow_tts(data_result[xh])
                    if audio_data:
                        break
                except Exception as e:
                    logging.warning(f"TTS第{attempt+1}次尝试失败: {str(e)}")
                    time.sleep(1)
        
        if not audio_data:
                logging.error("TTS服务不可用")
                return Response("TTS service unavailable", status=503)

        logging.info(f"音频生成耗时: {time.time()-tts_start:.2f}s,音频大小%d",len(audio_data))

        # 7. 验证音频格式
        if len(audio_data) < 44 or audio_data[0:4] != b'RIFF':
                logging.error("无效的WAV文件头")
                return Response("Invalid audio format", status=500)

        # 8. 返回响应
        total_time = time.time() - start_time
        logging.info(f"总处理时间: {total_time:.2f}s")
        return send_file(
                io.BytesIO(audio_data),
                mimetype='audio/wav',
                as_attachment=False
        )

    except requests.exceptions.Timeout:
        logging.error("上游API请求超时")
        return Response("Upstream service timeout", status=504)
    except Exception as e:
        logging.error(f"处理失败: {str(e)}", exc_info=True)
        return Response(str(e), status=500)

def siliconflow_tts(text):
    """调用硅流语音合成API（增强版）"""
    try:
        headers = {
            "Authorization": f"Bearer {SILICONFLOW_API_KEY}",
            "Content-Type": "application/json",
            "Accept": "audio/wav"
        }
        
        payload = {
            "model": "FunAudioLLM/CosyVoice2-0.5B",
            "input": text,  # 限制输入长度
            "voice": "FunAudioLLM/CosyVoice2-0.5B:alex",       # 简化的声音参数
            "response_format": "wav",
            "sample_rate": 16000,
            "speed": 1.0
        }

        response = requests.request("POST",TTS_API_URL, json=payload, headers=headers)
        
        
        
        # 验证音频内容
        if response.headers.get('Content-Type') != 'audio/wav':
            raise ValueError("Invalid content type")
            
        if len(response.content) < 1024:
            raise ValueError("Audio data too small")
            
        return response.content
        
    except Exception as e:
        logging.error(f"TTS错误: {str(e)}")
        if hasattr(e, 'response') and e.response:
            logging.error(f"错误响应: {e.response.text[:200]}")
        raise

if __name__ == '__main__':
    app.run(
        host='0.0.0.0', 
        port=5000,
        threaded=True,
        use_reloader=False  # 避免双进程干扰
    )